package cs206proj.part3;

/**
 * A Dictionary interface (similar to java.util.Map interface)
 * It is a set of (key, element) pairs where key is used to search for an 
 * element in the dictionary.  Can store elements with the same key value.
 * The key field of the (element, key) pair must override the hashCode()
 * method of the Object class and return its 32 bit integer hashcode, and
 * must override the equals method of the Object class.
 *
 * @author Tia Newhall
 *
 * Copyright (c) 2000 Tia Newhall, Lisa Meeden
 */
import java.util.*;

public interface Dictionary<K,V> {

    /**
     * Returns an object with a matching key
     * @param key the key value to find
     * @return an object with a matching key
     */
    public V get(K key); 

    /**
     * Add a new (key,element) pair to the dictionary
     * @param key the key value used to search for the element
     * @param element the data value to store
     */
    public void put(K key, V value); 

    /**
     * Returns an object with a matching key and remove it from the Dictionary
     * @param key the key value to find
     * @return an object with a matching key that was removed, or null if no 
     * match
     */
     public V remove(K key); 

    /**
     * Returns an Iterator over all dictionary objects with a matching key 
     * @param key the key value to find
     * @return an object with a matching key
     */
    public Iterator<V> getAll(K key); 

    /**
     * Returns an Iterator over all dictionary objects with a matching key 
     * and removes them from the dictionary
     * @param key the key value to find
     * @return an Iterator of all objects that were removed, or null if no match
     */
     public Iterator<V> removeAll(K key); 

    /**
     * Returns the number of elements in the dictionary
     * @return the number of elements in the dictionary
     */
    public int size(); 

    /**
     * Returns true if this dictionary is empty
     * @return true if this dictionary is empty
     */
    public boolean isEmpty(); 

    /**
     * Returns true if this dictionary contains an object with the specified key
     * @param key the key to search for
     * @return true if this dictionary contains an object with the specified key
     */
    public boolean containsKey(K key); 

    /**
     * Returns true if this dictionary contains an matching object 
     * @param element the object to find
     * @return true if this dictionary contains an matching object 
     */
    public boolean containsElement(V element); 

    /**
     * Returns an iterator over all keys
     * @return an iterator over all keys
     */
    public Iterator<K> keys(); 

    /**
     * Returns an iterator over all elements
     * @return an iterator over all elements
     */
    public Iterator<V> elements(); 



}
