package cs206proj.part3;

/**
 * HREFScanner class that given a url, opens the url's file and 
 * scans through it returning href strings 
 * To use:
 *    the constructor Scanner(url_name, mode) with the name of the 
 *    url to scan then call getNextToken() to get the next href token in the 
 *    file.  If mode is set to WEBPAGES then only tokens of the form:
 *     "http://..." or "path/file.html" are valid href tokens
 *     Relative pathnames ("path/file.html" are converted to absolute 
 *     form ("url_name/path/file.html" and returned as tokens. 
 *    If mode is set to ALLHREFS then any href string is returned.  
 *    (e.g. if href="temp.ps.Z" is the next href parsed in webpage, then the 
 *     "url_name/temp.ps.Z" is returned by next call to getNextToken())
 *  Currently only the WEBPAGES option is supported
 *  @author Tia Newhall
 *
 * Copyright (c) 2000 Tia Newhall, Lisa Meeden
 */
import java.io.*;

public class HREFScanner {

	private Scanner scanner;
	private String url;
	private int mode;
	private String href_token;
	private boolean was_read;
	public static int WEBPAGES = 0;
	public static int ALLHREFS = 1;

	/**
	 * Constructor takes input string representing a url name tries to open the
	 * file, and initializes the scanner
	 */

	public HREFScanner(String url, int mode) throws Exception,
			IOException {
		this.mode = mode;
		this.url = url;
		this.href_token = null;
		this.was_read = true;
		this.scanner = new Scanner(url, Scanner.SCANURL);
	}
	

	/**
	 * return true when there are no more tokens in the file or string
	 * 
	 * @return true when there are no more tokens in the file or string
	 */
	public boolean atEndOfFile() throws Exception {

		// there is a valid token in href_token that has not been read
		if (!was_read) {
			return false;
		}
		// otherwise check to see if there is still a valid href_token
		// in this file
		while (!scanner.atEndOfFile() && was_read) {
			// This function really does all the work
			boolean done = false;
			String link = null;
			while (!scanner.atEndOfFile() && !done) {
				String token = scanner.getNextToken();
				if ((token.toUpperCase()).equals("HREF")) {
					link = new String();
					boolean start = false;
					while (!scanner.atEndOfFile() && !done) {
						token = scanner.getNextToken();
						if (start) {
							if (token.equals("\"")) {
								done = true;
							} else {
								link = link + token;
							}
						} else if (token.equals("\"")) {
							start = true;
						}
					}
				}
			}
			if (done) {
				href_token = hrefToUrl(link, url);
				if (href_token != null)
					was_read = false;
			}
		}
		return was_read;
	}

	private String hrefToUrl(String link, String current_url) throws Exception {

		String link_url = new String();

		Scanner scanner = new Scanner(link, Scanner.SCANSTRING);
		if (!scanner.atEndOfFile()) {
			String token = scanner.getNextToken();
			if (token.toUpperCase().equals("HTTP")) {
				// get rid of the leading http:// stuff
				int count = 0;
				while (!scanner.atEndOfFile() && (count < 3)) {
					token = scanner.getNextToken();
					count++;
				}
				// if this is a messed-up link return null
				if (count < 3) {
					return null;
				}
				boolean found_domain = false;
				while (!scanner.atEndOfFile() && !found_domain) {
					token = scanner.getNextToken();
					link_url += token;
					if (token.equals("/")) {
						found_domain = true;
					}
				}
				// if this is not a local link return null
				/*if (!link_url.equals(ABSOLUTE)) {
					return null;
				}*/
				while (!scanner.atEndOfFile()) {
					token = scanner.getNextToken();
					link_url += token;
				}
				return link_url;
			} else { // check to see if this is a valid relative link
				boolean fix = false;
				StringBuffer ends_in_html = new StringBuffer(current_url);
				int len = ends_in_html.length();
				if (len > 5) {
					String suffix = ends_in_html.substring((len - 4));
					if (suffix.toUpperCase().equals("HTML")) {
						int last_bar = -1;
						for (int i = 0; i < len; i++) {
							if (ends_in_html.charAt(i) == '/') {
								last_bar = i;
							}
						}
						if (last_bar >= 0) {
							link_url = ends_in_html.substring(0, last_bar)
									+ "/" + token;
							fix = true;
						}
					}
				}
				if (!fix) {
					link_url = current_url + "/" + token;
				}
				// System.out.println("GETNEXTTOKEN: fixedlink_url = " +
				// link_url);
				while (!scanner.atEndOfFile()) {
					token = scanner.getNextToken();
					link_url += token;
				}
				// if the url doesn't end in .html then its not valid
				if (!(token.toUpperCase().equals("HTML"))) {
					return null;
				}

				// now remove all ../ from the link_url
				StringBuffer removedots = new StringBuffer(link_url);
				StringBuffer result = new StringBuffer();
				for (int i = 0; i < removedots.length(); i++) {
					char next = removedots.charAt(i);
					if (next == '.') {
						int onedot = 1;
						while ((i < removedots.length()) && (next == '.')) {
							i++;
							next = removedots.charAt(i);
							if ((onedot == 1) && (next != '.')) {
								result.append('.');
								result.append(next);
							}
							onedot++;
						}
						// we had a ../, need to remove previous path
						// component up to /
						if (onedot > 2) {
							int prev_end = -1, end = -1;
							for (int j = 0; j < result.length(); j++) {
								if (result.charAt(j) == '/') {
									prev_end = end;
									end = j;
								}
							}
							if (prev_end >= 0) {
								String temp2 = result.substring(0,
										(prev_end + 1));
								result = new StringBuffer(temp2);
							}
						}
					} else {
						result.append(next);
					}
				}

				// System.out.println("IN GETNEXTTOKEN: link_url = " + link_url
				// +
				// " result = " + result);
				return result.toString();
			}
		}
		return null;
	}

	/**
	 * return the next token in the file or string
	 * 
	 * @return the next token in the file or string
	 */
	public String getNextToken() throws Exception {
		if (was_read) {
			if (atEndOfFile()) {
				return null;
			}
		}
		was_read = true;
		return href_token;
	}
	
	
	/** Testing code for the HREFScanner
	 * 
	 */
	public static void main(String[] args) {
		try {
			String url = "http://www.brynmawr.edu";
			HREFScanner scanner = new HREFScanner(url, HREFScanner.WEBPAGES);
			while (!scanner.atEndOfFile()) {
				System.out.println(scanner.getNextToken());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
