package cs206proj.part3;

import java.awt.EventQueue;

import javax.swing.JFrame;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JSplitPane;
import javax.swing.JEditorPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/** Simple WebBrowser created for CS 206 Project
 * 
 * @author Eric Eaton
 * @date 2011-11-17
 *
 */
public class WebBrowser {

	private JFrame frame;
	private JTextField textboxURL;    // this field holds the URL
	private JTextField textboxSearch; // this field holds the search query
	private JEditorPane paneWebpage;  // the display pane for the webpage
	private JEditorPane paneSearchResults;  // the display pane for the search results
	
	// the message to display by default in the url box
	private String defaultURLMessage = "Enter URL";
	// the message to display by default in the search box
	private String defaultSearchMessage = "Search Query";

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					WebBrowser window = new WebBrowser();
					window.frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public WebBrowser() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		
		//////// the application frame
		frame = new JFrame();
		frame.setBounds(100, 100, 450, 300);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().setLayout(new BorderLayout()); // using a border layout, which has North and Center components
		
		
		//////// top URL/search bar
		JPanel topPanel = new JPanel();
		frame.getContentPane().add(topPanel, BorderLayout.NORTH);  // located in the north of the frame
		topPanel.setLayout(new GridBagLayout());
		
		// URL textfield
		textboxURL = new JTextField();
		textboxURL.setText(defaultURLMessage);
		GridBagConstraints layoutConstraints = new GridBagConstraints();
		layoutConstraints.fill = GridBagConstraints.HORIZONTAL;  // resize in both x and y directions
		layoutConstraints.weightx = 1.0;                         // x-dimension given a relative weight of 1
		topPanel.add(textboxURL, layoutConstraints);
		
		// URL get button
		JButton btnGetURL = new JButton("Get URL");
		layoutConstraints.fill = GridBagConstraints.NONE; // don't resize the component
		layoutConstraints.weightx = 0;                    // don't give any weight to the x dimension on resizing (since it doesn't resize)
		topPanel.add(btnGetURL, layoutConstraints);
		
		// search textfield
		textboxSearch = new JTextField();
		textboxSearch.setText(defaultSearchMessage);
		layoutConstraints.fill = GridBagConstraints.BOTH; // resize in both x and y directions
		layoutConstraints.weightx = 0.5;                  // half the x-dimension of the URL textbox
		topPanel.add(textboxSearch,layoutConstraints);
		
		// search button
		JButton btnSearch = new JButton("Search");
		layoutConstraints.fill = GridBagConstraints.NONE; // don't resize the component
		layoutConstraints.weightx = 0;                    // don't give any weight to the x dimension on resizing (since it doesn't resize)
		topPanel.add(btnSearch, layoutConstraints);
		
		
		////// main display panel
		JSplitPane splitPane = new JSplitPane();             // the split pane allows you to separate two components
		splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT); // separate the two components vertically
		splitPane.setResizeWeight(0.6);                      // controls the weight distribution between the components (0.5 is even)
		frame.getContentPane().add(splitPane);
		
		
		/////// webpage pane
		paneWebpage = new JEditorPane();
		paneWebpage.setEditable(false);                     // user can't edit the pane
		JScrollPane webpageScrollPane = new JScrollPane(paneWebpage);  		//Put the webpage pane in a scroll pane
		webpageScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		splitPane.setRightComponent(webpageScrollPane);
		splitPane.setLeftComponent(webpageScrollPane);
		
		
		///////// search results pane
		paneSearchResults = new JEditorPane();
		paneSearchResults.setEditable(false); 				// user can't edit the pane
		JScrollPane searchResultsScrollPane = new JScrollPane(paneSearchResults);  	//Put the search results pane in a scroll pane
		searchResultsScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		splitPane.setRightComponent(searchResultsScrollPane);

		
		////// add action listeners
		
		// listener to load the webpage when the GetURL button is pressed
		btnGetURL.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e)
            {
				String urlString = textboxURL.getText();
				// if the default message hasn't been changed, just exit without doing anything
				if (urlString.equalsIgnoreCase(defaultURLMessage))
					return;
				
				// retrieve the webpage
				try {
	            	paneWebpage.setPage(urlString);
				} catch (Exception e1) {
					paneWebpage.setContentType("text/html");
					paneWebpage.setText("<html>Could not load "+urlString+"</html>");
				}
            }
        });
	}

}
